﻿using Infragistics.Olap.FlatData;
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;

namespace CASE_178896
{
    public partial class Form1 : Form
    {
        public Form1()
        {
            InitializeComponent();
        }

        private void Form1_Load(object sender, EventArgs e)
        {
            FlatDataSource ds = InitializePivotData();
            // Generate cubes and call "InitializeAsync" method passing the control the data is bound to.
            var parameters = new CubeGenerationParameters();
            ds.GenerateCube(parameters);
            ds.InitializeAsync(ultraPivotGrid1);
            // Bind the PivotGrid and DataSelector to data.
            ultraPivotGrid1.DataSource = ds;
            olapDataSelector1.DataSource = ds;
            // Optinally, set compact mode on PivotGrid
            ultraPivotGrid1.RowHeaderLayout = Infragistics.Win.UltraWinPivotGrid.RowHeaderLayout.Compact;
        }

        public FlatDataSource InitializePivotData()
        {
            // Initialize the data
            SampleFlatData data = new SampleFlatData();
            IEnumerable<SportsData> itemSource = data.InitSportsData();
            // Use "FlatDataSourceInitialSettings" object to initialize Hierarchies and Measures
            var settings = new FlatDataSourceInitialSettings();
            settings.Rows = "[SportsData].[Nation], [SportsData].[Date]";
            settings.Columns = "[SportsData].[Sport]";
            settings.Measures = "[Measures].[Budget]";
            // Create an instance of the FlatDataSource
            var ds = new FlatDataSource(itemSource, typeof(SportsData), settings);
            // Initialize Hierarchy Descriptor
            ds.InitializeHierarchyDescriptor += ds_InitializeHierarchyDescriptor;
            // Initialize Measure Descriptor
            ds.InitializeMeasureDescriptor += ds_InitializeMeasureDescriptor;
            // Aggregate measures to produce totals
            ds.AggregateMeasure += ds_AggregateMeasure;
            return ds;
        }

        void ds_InitializeHierarchyDescriptor(object sender, InitializeHierarchyDescriptorEventArgs e)
        {
            LevelDescriptor level;
            if (e.HierarchyDescriptor.LevelDescriptors.TryGetItem("all", out level))
            {
                switch (e.HierarchyDescriptor.PropertyDescriptor.Name)
                {
                    case "Nation":
                        level.MemberProvider = item => "All Nations";
                        break;
                    case "City":
                        level.MemberProvider = item => "All Cities";
                        break;
                    case "Sport":
                        level.MemberProvider = item => "All Sports";
                        break;
                }
            }
        }

        void ds_InitializeMeasureDescriptor(object sender, InitializeMeasureDescriptorEventArgs e)
        {
            if (e.MeasureDescriptor.Name == "Budget")
            {
                e.MeasureDescriptor.DisplayFormat = "$#";
                e.MeasureDescriptor.Aggregation = MeasureAggregation.Average;
            }
        }

        void ds_AggregateMeasure(object sender, AggregateMeasureEventArgs e)
        {
            if (e.MeasureDescriptor.Name == "Budget")
            {
                double total = 0f;
                double count = 0f;
                foreach (object o in e.Items)
                {
                    var item = o as SportsData;
                    if (item != null)
                        total += item.Budget;
                    count += 1f;
                }
                e.Value = total / count;
                e.Handled = true;
            }
        }
    }
}
